/* Slide pma beneath C++ STL <map> for an easy persistent balanced tree.
   Copyright (C) 2019, 2022  Terence Kelly
   Contact:  tpkelly @ { acm.org, cs.princeton.edu, eecs.umich.edu }

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Affero General Public License for more details.

   You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.

   Compile as C++11.  To prevent trouble related to function pointers
   embedded in objects, it is prudent to forbid position independence
   via, e.g., the "-no-pie" flag of g++.
*/

#include <iostream>
#include <map>
#include "pma.h"

using namespace std;

#define EX(i) do { cerr << "pma_" i << ": " << pma_errno << endl; \
                   exit(EXIT_FAILURE); } while (0)

// "slide pma heap beneath": overload global operators new/delete
#define CK if (!r) EX("malloc"); return r
void * operator new      (size_t s) { void *r = pma_malloc(s); CK; }
void * operator new[]    (size_t s) { void *r = pma_malloc(s); CK; }
void   operator delete   (void * p) {           pma_free(p);       }
void   operator delete[] (void * p) {           pma_free(p);       }

int main(int argc, char *argv[]) {
  if (2 != argc) {
    cerr << "usage: " << argv[0] << " heapfile" << endl;
    return 1;
  }
  int r = pma_init(2, argv[1]);
  if (0 != r) EX("init");
  // if necessary create new STL <map>, store entry point in root
  typedef map<string, int> m_t;
  if (nullptr == pma_get_root()) pma_set_root(new m_t);
  m_t *SM = static_cast<m_t *>(pma_get_root());  // map entry point
  string w;
  while (cin >> w) {
    if ("[dump]" == w)
      for (m_t::iterator i = SM->begin(); i != SM->end(); ++i)
        cout << i->first << " : " << i->second << endl;
    else if (SM->find(w) == SM->end()) (*SM)[w] = 1;
    else                               (*SM)[w]++;
  }
  return 0;
}

