#!/bin/csh -f

# Test persistent memory allocator with C++ STL <map> (driver).
# Copyright (C) 2022  Terence Kelly
# Contact:  tpkelly @ { acm.org, cs.princeton.edu, eecs.umich.edu }
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


set b  = test7
set bf = pma_backing_file

alias clean 'rm -f $b $bf'

clean

if ("clean" == $1) then
    exit
endif

# To prevent trouble related to function pointers embedded in objects
# on the persistent heap, prevent position-idependent executables via
# g++'s "-no-pie" flag.
echo ; echo '= = = = = = = = = = compile for warnings' ; echo
g++ -std=c++11 -Wall -Wextra -no-pie -O3 -o $b $b.c++ pma.c
echo ; echo '= = = = = = = = = = compile for debugging' ; echo
g++ -std=c++11 -Wall -Wextra -no-pie -g  -o $b $b.c++ pma.c

echo ; echo '= = = = = = = = = = run first time on newborn persistent heap' ; echo
# create backing file; must be multiple of system page size
truncate -s 409600 $bf
(echo aaa; echo bbb; echo ccc; echo '[dump]') | ./$b $bf

echo ; echo '= = = = = = = = = = run second time, verifying persistence' ; echo
(echo ccc; echo zzz; echo aaa; echo '[dump]') | ./$b $bf

echo ; echo '= = = = = = = = = = run to memory exhaustion' ; echo
gawk 'BEGIN{while(1)print rand();}' | ./$b $bf

echo

clean

